@@ -88,17 +88,18 @@ def check_config(config):
     if (config.get('method', '') or '').lower() == 'rc4':
         logging.warn('warning: RC4 is not safe; please use a safer cipher, '
                      'like AES-256-CFB')
-    if (config.get('timeout', 600) or 600) < 100:
+    if (int(config.get('timeout', 300)) or 300) < 100:
         logging.warn('warning: your timeout %d seems too short' %
-                     config.get('timeout'))
-    if (config.get('timeout', 600) or 600) > 600:
+                     int(config.get('timeout')))
+    if (int(config.get('timeout', 300)) or 300) > 600:
         logging.warn('warning: your timeout %d seems too long' %
-                     config.get('timeout'))
+                     int(config.get('timeout')))
 
 
 def print_local_help():
     print '''usage: sslocal [-h] -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
-                -l LOCAL_PORT -k PASSWORD -m METHOD [-c config] [--fast-open]
+                -l LOCAL_PORT -k PASSWORD -m METHOD [-t TIMEOUT] [-c CONFIG]
+                [--fast-open]
 
 optional arguments:
   -h, --help            show this help message and exit
@@ -108,6 +109,7 @@ optional arguments:
   -l LOCAL_PORT         local port
   -k PASSWORD           password
   -m METHOD             encryption method, for example, aes-256-cfb
+  -t TIMEOUT            timeout in seconds
   -c CONFIG             path to config file
   --fast-open           use TCP_FASTOPEN, requires Linux 3.7+
 '''
@@ -115,7 +117,7 @@ optional arguments:
 
 def print_server_help():
     print '''usage: ssserver [-h] -s SERVER_ADDR -p SERVER_PORT -k PASSWORD
-                -m METHOD [-c config] [--fast-open]
+                -m METHOD [-t TIMEOUT] [-c CONFIG] [--fast-open]
 
 optional arguments:
   -h, --help            show this help message and exit
@@ -123,6 +125,7 @@ optional arguments:
   -p SERVER_PORT        server port
   -k PASSWORD           password
   -m METHOD             encryption method, for example, aes-256-cfb
+  -t TIMEOUT            timeout in seconds
   -c CONFIG             path to config file
   --fast-open           use TCP_FASTOPEN, requires Linux 3.7+
   --workers WORKERS     number of workers, available on Unix/Linux
